from math import floor

from features.moving_feature import moving_feature
from features.standard_deviation import standard_deviation
from korpus import create_bewegung

CLASS_JOGGEN = 1
CLASS_KNIEBEUGE = -1
CLASS_GEHEN = 0
CLASS_JUMPINGJACK = 2

TRAINING_DATA_PERCENTAGE = 0.9

DATA_LIMIT = 1000
DATA_LIMIT_PER_TYPE = floor(DATA_LIMIT / 2)


def get_features(feature_function, window_size, bewegungs_name, messung, sensor, personen: list):
    values = []

    for person in personen:
        bewegung = create_bewegung(bewegungs_name, gruppe=person[0], person=person[1])
        values += list(bewegung.messungen[messung].sensoren[sensor].werte)

    return moving_feature(feature_function, window_size, values)


def classification_evaluation(window_size=30, func=standard_deviation):
    messung = 0
    sensor = 0
    joggen_values = create_bewegung('joggen', person=1).messungen[messung].sensoren[sensor].werte
    kniebeuge_values = create_bewegung('gehen', person=1).messungen[messung].sensoren[sensor].werte
    jj_values = create_bewegung('jumpingjack', person=1).messungen[messung].sensoren[sensor].werte
    joggen_values2 = create_bewegung('joggen', person=2).messungen[messung].sensoren[sensor].werte
    kniebeuge_values2 = create_bewegung('gehen', person=2).messungen[messung].sensoren[sensor].werte
    jj_values2 = create_bewegung('jumpingjack', person=2).messungen[messung].sensoren[sensor].werte

    # Berechne die Standardabweichung (Bei Kniebeugen gering, bei Joggen hoch)
    joggen_feature = moving_feature(func, window_size, list(joggen_values) + list(joggen_values2))
    kniebeuge_feature = moving_feature(func, window_size, list(kniebeuge_values) + list(kniebeuge_values2))
    jj_feature = moving_feature(func, window_size, list(jj_values) + list(jj_values2))

    return joggen_feature, kniebeuge_feature, jj_feature


def label_testdata(features, label, data_limit):
    # Wir nehmen nur DATA_LIMIT an Daten (sonst ist K-Nearest-Neighbors zu langsam)
    features = features[: min(data_limit, len(features))]

    # Wandel Liste an Merkmalen in einzelne Merkmalsvektoren um
    feature_vector = ([element] for element in features)

    # Weise den Trainingsdaten eine Klasse zu
    data = list(zip(feature_vector, [label] * len(features)))

    # Wir nehmen 90 % der Testdaten zum Trainieren und 10 % zum Testen
    delimiter = floor(len(features) * TRAINING_DATA_PERCENTAGE)

    training_data = data[:delimiter]
    test_data = data[delimiter:]

    return test_data, training_data


def get_evaluation_data(window_size, feature_list: []):
    joggen_vector = None
    kniebeugen_vector = None
    jj_vector = None

    for feature in feature_list:
        # Hole die aus den Sensordaten berechneten Merkmale
        joggen_feature, kniebeuge_feature, jj_feature = classification_evaluation(window_size, feature)

        if not joggen_vector:
            joggen_vector = [[] for _ in joggen_feature][: min(DATA_LIMIT_PER_TYPE, len(joggen_feature))]
            kniebeugen_vector = [[] for _ in kniebeuge_feature][: min(DATA_LIMIT_PER_TYPE, len(kniebeuge_feature))]
            jj_vector = [[] for _ in jj_feature][: min(DATA_LIMIT_PER_TYPE, len(jj_feature))]

        # Wir nehmen nur DATA_LIMIT an Daten (sonst ist K-Nearest-Neighbors zu langsam)
        joggen_feature = joggen_feature[: min(DATA_LIMIT_PER_TYPE, len(joggen_feature))]
        kniebeuge_feature = kniebeuge_feature[: min(DATA_LIMIT_PER_TYPE, len(kniebeuge_feature))]
        jj_feature = jj_feature[: min(DATA_LIMIT_PER_TYPE, len(jj_feature))]
        for i in range(len(joggen_feature)):
            joggen_vector[i].append(joggen_feature[i])
        for i in range(len(kniebeuge_feature)):
            kniebeugen_vector[i].append(kniebeuge_feature[i])
        for i in range(len(jj_feature)):
            jj_vector[i].append(jj_feature[i])

    # Weise den Trainingsdaten eine Klasse zu
    # 0 = Kniebeuge, 1 = Joggen
    training_data_joggen = list(zip(joggen_vector, [CLASS_JOGGEN] * len(joggen_vector)))
    training_data_kniebeuge = list(zip(kniebeugen_vector, [CLASS_KNIEBEUGE] * len(kniebeugen_vector)))
    training_data_jj = list(zip(jj_vector, [CLASS_JUMPINGJACK] * len(jj_vector)))

    # Wir nehmen 90 % der Testdaten zum Trainieren und 10 % zum Testen
    delimiter_joggen = floor(len(joggen_vector) * TRAINING_DATA_PERCENTAGE)
    delimiter_kniebeuge = floor(len(kniebeugen_vector) * TRAINING_DATA_PERCENTAGE)
    delimiter_jj = floor(len(jj_vector) * TRAINING_DATA_PERCENTAGE)

    training_data = training_data_joggen[:delimiter_joggen] + training_data_kniebeuge[
                                                              :delimiter_kniebeuge] + training_data_jj[:delimiter_jj]
    test_data = training_data_joggen[delimiter_joggen:] + training_data_kniebeuge[
                                                          delimiter_kniebeuge:] + training_data_jj[delimiter_jj:]

    return test_data, training_data
